// Drill blind vias code
// Diyouware TwinTeeth mini-PCB factory,
// Copyright 2015 by V.Aylln.
//

/**
/* write_blindvias_header() - writes the gcode header
**/
void write_blindvias_header() {
	output(blindvias_filename,"wt") {
		printf(
			//"G28 ; home axis\n"
			"G90 ; use absolute coordinates\n"
			"G21 ; set units to millimeters\n"
			"G0 Z%0.3f F%d\n", FILLVIA_Z_HIGH , FILLVIA_FEED_RATE
			);
	}
}
/**
/* write_blindvias_item() - writes a blindvias coordinates item
**/
void write_blindvias_item(int i) {
	output(blindvias_filename,"at") {
		printf("G81 X%.2f Y%.2f Z%.2f ", u2mm(x[i]), u2mm(y[i]), FILLVIA_Z_DOWN + (abs(FILLVIA_DEPTH)*-1)); // move to new position, above board
		printf("R%.2f D%.2f ", FILLVIA_Z_UP, FILLVIA_Z_DOWN);  // ready to drill.
		if (DRILL_DWELL > 0)
			printf("P%.1f ",FILLVIA_DWELL);  // Wait given milliseconds
		printf("F%d\n",FILLVIA_FEED_RATE_Z);  // Feedrate
	}
}
/**
/* write_blindvias_trailer() - writes the end of the gcode file
**/
void write_blindvias_trailer(){
	output(blindvias_filename,"at") {
	printf("G0 Z%0.3f F%d\n", FILLVIA_Z_HIGH , FILLVIA_FEED_RATE);
	printf("M84 ; shut down motors, done\n");
	}
}

/**
/* blindvias_genfiles() - generates the dispensing gcode files
**/
void blindvias_genfiles(int n) {
	// Sort data to optimize top/bottom, coord.
	sort(n, index, y, x);  
	// write items to file
	for (int i = 0; i < n; i++) {
		write_blindvias_item(index[i]);
	}
}
/**
/* blindvias_check() - Check what size of vias has the board
**/
string blindvias_check(){

	text_buffer = "";
	int via_index = 0;
	real via_size_mm[];
	real drill_mm;
	int found = 0;

	if (board) {
		board(B) {
			B.signals(S) S.vias(V) {
				drill_mm = u2mm(V.drill);
				found = 0;
				for (int i = 0; i < via_index; i++){
					if (via_size_mm[i] == drill_mm)
						found = 1;
					break;  // found
				}
				if (found == 0){
					// Not found, include in list
					via_size_mm[via_index] = drill_mm;
					via_index++;
				}
			}
		}

		// Sort data
		sort(via_index, vindex, via_size_mm);  

		for (int i = 0; i < via_index; i++){
			sprintf(temp, "%.3f mm\n",via_size_mm[vindex[i]]);
			text_buffer = text_buffer + temp;
		}
	}
	return text_buffer;
}
/**
/* blindvias_main() - main solder paste dispensing code
**/
void blindvias_main(){

	int n = 0;

	if (board) {
		blindvias_filename = board_name + ".drillblindvias.gcode";
		write_blindvias_header();
		board(B) {
			// Collect all vias data and store it in memory
			B.signals(S) {
				S.vias(V) {
					x[n] = V.x;
					y[n] = V.y;
					n++;
				}
			} 
			// Gen the files
			blindvias_genfiles(n);

		} // Signals
		//  Write trailer
		write_blindvias_trailer();
	} // Board
}