// Fill blind vias code
// Diyouware TwinTeeth mini-PCB factory,
// Copyright 2015 by V.Aylln.
//

/**
/* write_fillvias_header() - writes the gcode header
**/
void write_fillvias_header() {
	output(fillvias_filename,"wt") {
		printf(
			//"G28 ; home axis\n"
			"M83 ; use relative distances for filling\n"
			"G90 ; use absolute coordinates\n"
			"G21 ; set units to millimeters\n"
			"G92 E0 ;reset extruder\n"
			"G0 Z%0.3f F%d\n", FILLVIA_Z_HIGH , FILLVIA_FEED_RATE
			);
	}
}
/**
/* write_fillvias_item() - writes a fillvias coordinates item
**/
void write_fillvias_item(int i) {

	output(fillvias_filename,"at") {
		printf("G81 X%.2f Y%.2f Z%.2f E%.2f ", u2mm(x[i]), u2mm(y[i]),FILLVIA_Z_DOWN + (abs(FILLVIA_DEPTH)*-1),e[i]); // Move to new position, above board
		printf("R%.2f D%.2f ", FILLVIA_Z_UP, FILLVIA_Z_DOWN); 
		printf("F%d ",FILLVIA_FEED_RATE_Z);  // Feed rate
		printf("W%d\n",FILLVIA_EXTRUDER_FEEDRATE);  // Flow (Filling rate)
		if (FILLVIA_RETRACTION_LEN > 0)
			printf("G1 E-%.5f F%d\n", FILLVIA_RETRACTION_LEN, FILLVIA_EXTRUDER_FEEDRATE);  // Retract the paste
	}
}
/**
/* write_fillvias_trailer() - writes the end of the gcode file
**/
void write_fillvias_trailer(){
	output(fillvias_filename,"at") {
		printf("G0 Z%0.3f F%d\n", FILLVIA_Z_HIGH , FILLVIA_FEED_RATE);
		printf("M84 ; shut down motors, done\n");
	}
}

/**
/* fillvias_genfiles() - generates the dispensing gcode files
**/
void fillvias_genfiles(int n) {
	// Sort data to optimize top/bottom, coord.
	sort(n, index, y, x);  
	// write items to file
	for (int i = 0; i < n; i++) {
		write_fillvias_item(index[i]);
	}
}
/**
/* fillvias_main() - main solder paste dispensing code
**/
void fillvias_main(){

	int n = 0;

	if (board) {
		fillvias_filename = board_name + ".fillvias.gcode";
		write_fillvias_header();
		board(B) {
			// Collect all vias data and store it in memory
			B.signals(S) {
				S.vias(V) {
					x[n] = V.x;
					y[n] = V.y;
					// Calculate the srynge move in mm. We use the via volume and the area of the plunger.
					e[n] = real(pow(real(u2mm(V.drill)/2.0),2.0)*FILLVIA_PCB_THICKNESS)/pow(real(FILLVIA_PLUNGER_DIAMETER/2.0),2.0);
					n++;
				}
			} 
			// Gen the files
			fillvias_genfiles(n);

		} // Signals
		//  Write trailer
		write_fillvias_trailer();
	} // Board
}